# RebuildIndexFilter
#
# Release 0.01 (Sep 10, 2008)
# Release 0.02 (Sep 12, 2008) change to checkbox from checkbox and change script-name check.
# Release 0.03 (Sep 15, 2008) check approved,unapproved,junk,unjunk and fixed bug.
# Release 0.04 (Sep 16, 2008) fix bug.
# Release 0.05 (Sep 16, 2008) fix bug.
# Release 0.06 (Sep 17, 2008) add schdule task.
#
# This software is provided as-is. You may use it for commercial or
# personal use. If you distribute it, please keep this notice intact.
#
# Copyright (c) 2004-2008 Yujiro Araki

package MT::Plugin::RebuildIndexFilter;

use strict;
use warnings;
use MT;
use base qw( MT::Plugin );
use MT::RebuildIndexFilter;

our $VERSION = '0.06';
our $SCHEMA_VERSION = '0.1';

our $schedule_entry = 0;
our $schedule_page = 0;

my $plugin = __PACKAGE__->new ({
    id   => 'rebuildindexfilter',
    name => 'RebuildIndexFilter',
    description => '<MT_TRANS phrase="Filter index template when rebuild.">',
    doc_link => 'http://www.koikikukan.com/archives/2008/09/10-005555.php',
    author_name => 'Yujiro Araki',
    author_link => 'http://www.koikikukan.com/',
    l10n_class => 'RebuildIndexFilter::L10N',
    version => $VERSION,
    schema_version => $SCHEMA_VERSION,
});
MT->add_plugin($plugin);

sub init_registry {
    my $plugin = shift;
    $plugin->registry({
        object_types => {
            'template_ext' => 'MT::RebuildIndexFilter',
        },
        callbacks => {
             'build_file_filter' => \&rebuild_index_filter,
             'scheduled_post_published' => \&get_schedule,
             'MT::App::CMS::template_source.edit_template' => \&add_rebuild_event,
             'MT::App::CMS::template_param.edit_template' => \&get_rebuild_event,
             'cms_post_save.template' => \&save_rebuild_event,
             'cms_post_delete.template' => \&delete_rebuild_event,
        },
   });
}

sub rebuild_index_filter {
    my ($eh, %args) = @_;
    return 1 if $args{'ArchiveType'} ne 'index';
MT->instance->log("root1");
    my $id = $args{template}->id;
    my $rif = MT::RebuildIndexFilter->load({ template_id => $id });

    # no setting
    return 1 unless (defined $rif);
MT->instance->log("root2");
    my $rebuild_event = $rif->rebuild_event;

    # no check or all
    return 1 if ($rebuild_event eq '' || $rebuild_event eq 'all');
MT->instance->log("root3");
    my $script = $ENV{'SCRIPT_NAME'};

    my $type;
    my $app = MT->instance;
    my $admin_script = $app->config('AdminScript');

    if ((defined $admin_script) && ($script =~ /$admin_script/)) {
        my $q = $app->param;

        # rebuild
        if ($q->param('__mode') eq 'rebuild') {

            # on template page
            return 1 if ($q->param('return_args') =~ /_type=template/);

            # on rebuild display
            return 1 if ($q->param('type') =~ /index/)
        }

        # operation on list_comment
        if ($q->param('return_args') =~ /__mode=list_comment/) {
            return 1 if ($q->param('return_args') =~ /approved=1|junked=1/) && ($rebuild_event =~ /comment/);
        }

        # operation on list_pings
        if ($q->param('return_args') =~ /__mode=list_pings/) {
            return 1 if ($q->param('return_args') =~ /approved=1|junked=1/) && ($rebuild_event =~ /ping/);
        }

        # operation on list_entry
        return 1 if ($q->param('return_args') =~ /__mode=list_entry/) && ($rebuild_event =~ /entry/);

        # operation on list_pages
        return 1 if ($q->param('return_args') =~ /__mode=list_pages/) && ($rebuild_event =~ /page/);

        $type = $q->param('_type');

        # operation on comment edit page.
        if ($type eq 'comment') {
            return 1 if $rebuild_event =~ /comment/;
        }

        # operation on ping edit page.
        if ($type eq 'ping') {
            return 1 if $rebuild_event =~ /ping/;
        }

        # edit entry or page
        $type = '';
        if ($q->param('return_args') eq '') {
            my $entry = MT::Entry->load($q->param('entry_id'));
            if ($entry) {
                $type = $entry->class;
            }
        } else {
            # new entry or page
            $type = $q->param('_type');
        }
        if ($type) {
            my @event = ('entry','page');
            for my $event (@event) {
                return 1 if ($event =~ /$type/) && ($rebuild_event =~ /$event/);
            }
        }
    }

    # post comment
    my $comment_script = $app->config('CommentScript');
    return 1 if (defined $comment_script) && ($script =~ /$comment_script/) && ($rebuild_event =~ /comment/);

    # receive ping
    my $trackback_script = $app->config('TrackbackScript');
    return 1 if (defined $trackback_script) && ($script =~ /$trackback_script/) && ($rebuild_event =~ /ping/);
$app->log("root4");
    # schedule task(perhaps)
    my $match;
    if (!$script) {
$app->log("root5");
$app->log("entry:".$schedule_entry);
$app->log("page:".$schedule_page);
        $match = 1 if ($schedule_entry && $rebuild_event =~ /entry/) || ($schedule_page && $rebuild_event =~ /page/);
    }
    $schedule_entry = 0;
    $schedule_page = 0;
    return 1 if $match;
$app->log("root6");
    return 0;
}

sub get_schedule {
    my ($eh, $mt, $entry) = @_;
    if ($entry) {
        my $type = $entry->class;
        $schedule_entry = 1 if ($type eq 'entry');
        $schedule_page = 1 if ($type eq 'page');
    }
    return 1;
}

sub add_rebuild_event {
    my ($cb, $app, $template) = @_;

    my $old = <<HERE;
    </mt:if>
</mt:if>
<mt:if name="template_group" like="(module|widget)">
HERE
    $old = quotemeta($old);

    my $label = $plugin->translate("Rebuild_event");
    my $all = $plugin->translate("All");
    my $entry = $plugin->translate("Entry");
    my $page = $plugin->translate("Page");
    my $comment = $plugin->translate("Comment");
    my $ping = $plugin->translate("Ping");

    my $new = <<HERE;
<script type="text/javascript">
function toggleAndUncheckForRebuildEvent(obj, target) {
    if (!obj || !target) {
        return;
    }
    var targetObj = getByID(target);
    if (!targetObj) {
        return;
    }
    if (obj.checked) {
//        targetObj.checked = true;
        targetObj.disabled = true;
    } else {
//        targetObj.checked = false;
        targetObj.disabled = false;
    }
}
function checkEventsForRebuildEvent(obj) {
    toggleAndUncheckForRebuildEvent(obj,'rebuild-event-entry');
    toggleAndUncheckForRebuildEvent(obj,'rebuild-event-page');
    toggleAndUncheckForRebuildEvent(obj,'rebuild-event-comment');
    toggleAndUncheckForRebuildEvent(obj,'rebuild-event-ping');
}
</script>
        <mtapp:setting
            id="rebuild_event"
            label="$label"
            content_class="field-content-text">
            <ul id="rebuild-events">
                <li><input type="checkbox" value="all" name="rebuild_event" id="rebuild-event-all" onclick="checkEventsForRebuildEvent(this)"<mt:if name="rebuild_event" eq="all"> checked="checked"</mt:if>> <label for="cache-event-all">$all</label></li>
                <li><input type="checkbox" value="entry" name="rebuild_event" id="rebuild-event-entry" <mt:if name="rebuild_event" like="entry"> checked="checked"</mt:if><mt:if name="rebuild_event" like="all"> disabled="disabled"</mt:if>> <label for="rebuild-event-entry">$entry</label></li>
                <li><input type="checkbox" value="page" name="rebuild_event" id="rebuild-event-page" <mt:if name="rebuild_event" like="page"> checked="checked"</mt:if><mt:if name="rebuild_event" like="all"> disabled="disabled"</mt:if>> <label for="rebuild-event-page">$page</label></li>
                <li><input type="checkbox" value="comment" name="rebuild_event" id="rebuild-event-comment" <mt:if name="rebuild_event" like="comment"> checked="checked"</mt:if><mt:if name="rebuild_event" like="all"> disabled="disabled"</mt:if>> <label for="rebuild-event-comment">$comment</label></li>
                <li><input type="checkbox" value="ping" name="rebuild_event" id="rebuild-event-ping" <mt:if name="rebuild_event" like="ping"> checked="checked"</mt:if><mt:if name="rebuild_event" like="all"> disabled="disabled"</mt:if>> <label for="rebuild-event-ping">$ping</label></li>
            </ul>
        </mtapp:setting>
    </mt:if>
</mt:if>
<mt:if name="template_group" like="(module|widget)">
HERE
    $$template =~ s/$old/$new/;
}

sub save_rebuild_event {
    my ($cb, $app, $obj, $org) = @_;
    my $id = $obj->id;
    if ($id) {
        my $rif = MT::RebuildIndexFilter->load({ template_id => $id });
        my $q = $app->param;
        my @rebuild_event = $q->param('rebuild_event');
        if (@rebuild_event) {
            unless (defined $rif) {
                $rif = MT::RebuildIndexFilter->new;
                $rif->template_id($id);
            }
            $rif->rebuild_event(join(',', @rebuild_event));
            $rif->save or die $rif->errstr;
        } else {
            if (defined $rif) {
                $rif->rebuild_event('');
                $rif->save or die $rif->errstr;
            }
        }
    }
    return 1;
}

sub delete_rebuild_event {
    my ($cb, $app, $obj) = @_;
    my $id = $obj->id;
    if ($id) {
        my $rif = MT::RebuildIndexFilter->load({ template_id => $id });
        return 1 unless defined $rif;
        $rif->remove;
    }
    return 1;
}

sub get_rebuild_event {
    my ($cb, $app, $param, $tmpl) = @_;
    my $id = $param->{id};
    if ($id) {
        my $rif = MT::RebuildIndexFilter->load({ template_id => $id });
        unless (defined $rif) {
            $param->{rebuild_event} = '';
        } else {
            $param->{rebuild_event} = $rif->rebuild_event;
        }
    }
}

1;
